package org.greenstone.gsdl3.core;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.action.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

// other java classes
import java.io.File;
import java.util.HashMap;
import java.util.Enumeration;

/** The default greenstone receptionist - needs some extra info for each page
 * @see WebReceptionist
 */
public class DefaultReceptionist extends TransformingReceptionist {
    
    protected void addExtraInfo(Element page) {
	// we want to add in the collection description for each page - cos
	// our default xslt needs this.
	super.addExtraInfo(page);
	
	Element page_request = (Element)GSXML.getChildByTagName(page, GSXML.PAGE_REQUEST_ELEM);
	System.out.println("add extra info, page request="+this.converter.getString(page_request));
	// is a collection defined?
	Element param_list = (Element)GSXML.getChildByTagName(page_request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	if (param_list==null) { // must be the original home page
	    System.out.println("DefaultReceptionist: no param list, assuming home page");
	    return;
	}
	Element coll_param = GSXML.getNamedElement(param_list, GSXML.PARAM_ELEM, GSXML.NAME_ATT, GSParams.COLLECTION);
	if (coll_param == null) {
	    System.out.println("DefaultReceptionist: coll param is null, returning");
	    return;
	}
	String action = page_request.getAttribute(GSXML.ACTION_ATT);
	if (action.equals("p")) {
	    System.out.println("DefaultReceptionist: page action, so returning");
	    return;
	}
	// we have a collection, and we are not page action. get the coll description
	
	String collection = coll_param.getAttribute(GSXML.VALUE_ATT);
	String lang = page_request.getAttribute(GSXML.LANG_ATT);
	Element coll_about_message = this.doc.createElement(GSXML.MESSAGE_ELEM);
	Element coll_about_request = GSXML.createBasicRequest(this.doc, GSXML.REQUEST_TYPE_DESCRIBE,  collection, lang);
	coll_about_message.appendChild(coll_about_request);
	
	Element coll_about_response_message = this.mr.process(coll_about_message);
	Element coll_about_response = (Element)GSXML.getChildByTagName(coll_about_response_message, GSXML.RESPONSE_ELEM);
	Element coll_description = (Element)GSXML.getChildByTagName(coll_about_response, GSXML.COLLECTION_ELEM);
	if (coll_description==null) { // may be a cluster
	    coll_description = (Element)GSXML.getChildByTagName(coll_about_response, GSXML.CLUSTER_ELEM);
	}
	if (coll_description == null) {
	    System.out.println("DefaultReceptionist: no collection description, returning");
	    return;
	}
	
	Element page_response = (Element)GSXML.getChildByTagName(page, GSXML.PAGE_RESPONSE_ELEM);
	Element new_coll_description = (Element)this.doc.importNode(coll_description, true);
	page_response.appendChild(new_coll_description);
	
	// now get the dispay info for the services
	Element service_list = (Element)GSXML.getChildByTagName(new_coll_description, GSXML.SERVICE_ELEM+GSXML.LIST_MODIFIER);
	if (service_list == null) {
	    System.out.println("DefaultReceptionist: no service list, returning");
	    // something weird has gone wrong
	    return;
	}

	NodeList services = service_list.getElementsByTagName(GSXML.SERVICE_ELEM);
	// we will send all the requests in a single message
	Element info_message = this.doc.createElement(GSXML.MESSAGE_ELEM);
	for (int i=0; i<services.getLength(); i++) {
	    Element c = (Element)services.item(i);
	    String name = c.getAttribute(GSXML.NAME_ATT);
	    String address = GSPath.appendLink(collection, name);
	    Element info_request = GSXML.createBasicRequest(this.doc,  GSXML.REQUEST_TYPE_DESCRIBE, address, lang);
	    info_message.appendChild(info_request);
	}

	Element info_response = (Element)this.mr.process(info_message);

	NodeList service_responses = info_response.getElementsByTagName(GSXML.RESPONSE_ELEM);
	// check that have same number of responses as collections
	if (services.getLength() != service_responses.getLength()) {
	    System.err.println("DefaultReceptionist Error: didn't get a response for each service - somethings gone wrong!");
	    // for now, dont use the metadata
	} else {
	    for (int i=0; i<services.getLength(); i++) {
		Element c1 = (Element)services.item(i);
		Element c2 = (Element)GSXML.getChildByTagName((Element)service_responses.item(i), GSXML.SERVICE_ELEM);
		if (c1.getAttribute(GSXML.NAME_ATT).equals(c2.getAttribute(GSXML.NAME_ATT))) {
		    //add the service data into the original response
		    GSXML.mergeElements(c1, c2);
		} else {
		    System.err.println("DefaultReceptionist Error: response does not correspond to request!");
		} 
		
	    }
	}
	
    }
}

